@@ -61,7 +61,7 @@ $simnetwork @themereports %simtitles %reporttitles %retrievedata
 $target %dowhat
 );
 
-$VERSION = '0.46.3'; # our $VERSION = '';
+$VERSION = '0.47.1'; # our $VERSION = '';
 $ABSTRACT = 'Sim::OPT it a tool for detailed metadesign. It manages parametric explorations through the ESP-r building performance simulation platform and performs optimization by block coordinate descent.';
 
 #################################################################################
@@ -726,7 +726,7 @@ sub callcase # IT PROCESSES THE CASES.
 	#	$dirfiles{descendcases}, $dirfiles{descendstruct} );
 	#}
 	
-	open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
+	#open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
 	open( TOSHELL, ">>$toshell" ) or die "Can't open $toshell: $!"; 
 	
 	#if ( ($countcase > 0) or ($countblock > 0) )
@@ -1016,9 +1016,8 @@ sub exe
 sub start
 {
 ###########################################
-print "THIS IS OPT.
-Copyright by Gian Luca Brunetti and Politecnico di Milano, 2008-15.
-{ DAStU Department, Polytechnic of Milan }
+print "THIS IS Sim::OPT,
+a morphing and optimizazion program.
 
 .  .  .  .  .  .  .  .  .  .  .  .  .
 
@@ -1034,7 +1033,6 @@ Please insert the name of a configuration file for OPT (Unix path)\n\n";
 		
 sub opt
 {
-	
 	###############################################################
 
 	
@@ -1042,17 +1040,17 @@ sub opt
 	# eval `cat $configfile`; # The file where the program data are
 	
 	require $configfile;
-	if ( not ( $outfile ) ) { $outfile = "$mypath/$file-$fileconfig-feedback.txt"; } 
+	#if ( not ( $outfile ) ) { $outfile = "$mypath/$file-$fileconfig-feedback.txt"; } 
 	if ( not ( $toshell ) ) { $toshell = "$mypath/$file-toshell.txt"; } 
 	
 	$tee = new IO::Tee(\*STDOUT, ">>$toshell"); # GLOBAL ZZZ
 
-#	if ($casefile) { eval `cat $casefile` or die; }
-#	if ($chancefile) { eval `cat $chancefile` or die; }
+#	if ($casefile) { eval `cat $casefile` or die( "$!" ); }
+#	if ($chancefile) { eval `cat $chancefile` or die( "$!" ); }
 
 	print "\nNow in Sim::OPT.\n";
 		
-	open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
+	#open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
 	open( TOSHELL, ">>$toshell" ) or die "Can't open $toshell: $!"; 
 		
 	#unless (-e "$mypath") 
@@ -1155,32 +1153,30 @@ Sim::OPT.
 
 =head1 DESCRIPTION
 
-Sim::OPT is a set of tools for detailed metadesign of buildings. Its main components are an optimization module pursuing optimization by overlapping block coordinate descent and a morphing module which morphs building models by propagation of constraints through the ESP-r building performance simulation platform. Other components are retrieval and reporting modules and a module emitting parallel coordinates plots in 3D.
+Sim::OPT is an optimization and morphing program that in principle may be used with any kind of simulation program that can receive text files as input and emit text files ad output. Some of its optimization modules (Sim::OPT, Sim::OPT::Descent) pursue optimization by overlapping block coordinate descent (Gauss-Seidel method). Another optimization module (Sim::OPT::Takechange), that can be used optionally, seeks for the least explored search paths when exploring not-much-exploited exploited search spaces. Sim::OPT morphing module (Sim::OPT::Morph) manipulates text files by targeting specific variables position and applying to them generic interpolation rules. Another module (Sim::OPT::Parcoord3d) is dedicated to reporting and creating parallel coordinates plots in 3D.
 
-A working knowledge of ESP-r (http://www.esru.strath.ac.uk/Programs/ESP-r.htm) is presently necessary to use most functionalitues of OPT. (ESP-r must be compiled for text (not graphics) to be managed by OPT.)
+At a more specific level, Sim::OPT is aimed to the detailed metadesign of buildings through the ESP-r building performance simulation platform. Because of that, a working knowledge of ESP-r (http://www.esru.strath.ac.uk/Programs/ESP-r.htm) is presently necessary to use most functionalitues of OPT. (Note that ESP-r must be compiled for text, not graphics, and for Linux.) Sim::OPT features several morphing functions mapping the functions of the ESP-r platform and can manage it not only by manipulating text input files, but also through the shell.
 
-To install OPT, the command <cpanm Sim::OPT> has to be issued. OPT can be loaded through the command <use Sim::OPT> in Perl. For that purpose, the batch file "opt" (which can be found packed in the "optw.tar.gz" file in "example" folder in this distribution) may be copied in a work directory and the command <opt> may be issued. That command will activate the OPT's functions, following the settings specified in a previously prepared configuration file. When launched, OPT will ask the path to that file. Its activity will start after receiving that information. 
+To install Sim::OPT, the command <cpanm Sim::OPT> has to be issued as a superuser. OPT can then be loaded through the command <use Sim::OPT> in a Perl repl. To ease the launch, the batch file "opt" (which can be found packed in the "optw.tar.gz" file in "example" folder in this distribution) may be copied in a work directory and the command <opt> may be issued. That command will activate the OPT's functions, following the settings specified in the configuration file, which had to be prepared in advance. When launched, OPT will ask the path to that file. Its activity will start after receiving that information. 
 That file must contain a suitable description of the operations to be accomplished pointing to an existing ESP-r model.
 
-In "optw.tar.gz" there is an example of OPT configuration file ("v.pl"). That file may be decompacted and the resulting folder ("optw") may be used as a work folder for OPT, in which the ESP-r models to be worked may reside. The "$mypath" variable in the configuration file must be set to that work directory.
+In "optw.tar.gz" there is an example of OPT configuration file for an ESP-r model and another one for EnergyPlus (another building performance simulation program). The defined configuration file may be copied in a chosen work folder for OPT, in which the ESP-r models to be worked may reside. The "$mypath" variable in the configuration file must be set to that work directory.
 
-To run OPT without making it launch ESP-r, the setting <$exeonfiles = "n";> should be specified in the configuration file. This may only be aimed to inspect the commands that OPTS would give to ESP-r through the shell. (In the variable "$toshell" the path to the file that will receive the commands in place of the shell can be specified.) This is only useful for testing the morphing instruction contained in a configuration file before launching a real batch of simulations. Note that in that case the search path will be different from that driven by simulation results. If simulations are not launched, the optimal instance  at each subspace search cannot indeed be selected. In its place, the base case will be kept by the program until completion. A sequential block search (Gauss-Seidel method) cannot indeed be run "dry".
+To run OPT without making it launch ESP-r, the setting <$exeonfiles = "n";> should be specified in the configuration file. This can be aimed to inspect the morphing commands that OPTS would give to the simulation program. But that way a sequential block search cannot be performed, because it cannot be run "dry", without retrieving any information about the results at the end of each search cycle.
 
 If $exeonfiles is set to "y", OPT will give instruction to ESP-r via shell to make it modify the building model in the specified steps. Then, if asked, it will run simulations, retrieve the results, extract some information from them and order it as requested.
 
 Besides an OPT configuration file, separate configuration files for propagation of constraints may be created. Those can be useful to give the morphing operations greater flexibility. Propagation of constraints can regard the geometry of a model, solar shadings, mass/flow network, and/or controls; and also, how those pieces of information affect each other and daylighting.
 
-The ESP-r model folders and the result files that will be created in a parametric search will be named as the base building model, numbers and other characters to described an instance. For example, the instance produced in the first iteration for a root model named "model" in a search constituted by 3 morphing phases and 5 iteration steps each will be named "model_1-1_2-1_3-1"; and the last one "model_1-5_2-5_3-5".
+The ESP-r model folders and the result files that will be created in a parametric search will be named as the base building model, numbers and other characters to describe an instance. For example, the instance produced in the first iteration for a root model named "model" in a search constituted by 3 morphing phases and 5 iteration steps each will be named "model_1-1_2-1_3-1"; and the last one "model_1-5_2-5_3-5".
 
 The structure of the block searches is described through the variable "@sweeps". Each case is listed inside square brackets. And each search subspace (block) in them is listed inside square brakets, nested in cases. For example: a sequence constituted by two brute force searches, one regarding parameters 1, 2, 3 and the other regarding parameters 1, 4, 5, 7 would be described with: @sweeps = ( [ [ 1, 2, 3 ] ] , [ [ 1, 4, 5, 7 ] ] ). And a block search with the first subspace regarding parameters 1, 2, 3 and the second regarding parameters 3, 4, 5, 6 would be described with: @sweeps = ( [ [ 1, 2, 3 ] , [ 3, 4, 5, 6 ] ] ). 
 
 The number of iterations to be taken into account for each parameter for each case is specified in the "@varinumbers" variable. To specifiy that the parameters of the last example are to be tried for three values (iterations) each, @varinumbers has to be set to ( { 1 => 3, 2 => 3, 3 => 3, 4 => 3, 5 => 3, 6 => 3 } ).
 
-Some functionalities of OPT are not specific to the ESP-r platform. Those functionalities are mainly contanined in the "Sim::OPT::Takechance" and "Sim::OPT::Parcoord3d" modules.
+Where Sim::OPT may be fit for the task? Where a certain exploration is complex and/or when it is to be confronted through decomposition, by dividing a problem in overallapping subproblems. In general, when approaching a certain problem manually, or in part manually and in part automatically, would take ages, and there aren't slick tools suitable to decomposition-based, simulation-based optimization available. When spending a day or two setting up a model may spare month of work.
 
-The "Sim::OPT::Takechance" module produces efficient search structures for block coordinate descent given some initialization blocks. A section of the configuration file for "Sim::OPT" is dedicated to the "Sim::OPT::Takechance" module.
-
-The "Sim::OPT::Parcoord3d module when receiving as input the data for a bi-dimensional parallel coordinate plot in a cvs file produces as output an Autolisp file that can be used from Autocad or an Intellicad-derived 3D CAD program to obtain a 3D parallel coordinate plot (that may be subsequently saved in dwg, dxf or other formats). A section of the configuration file for "Sim::OPT" is dedicated to the "Sim::OPT::Parcoord3d" module.
+Where Sim::OPT may not be fit for the task? In its present state, due to its absolute crude interface, for quick shots at small explorations. There are many refined and user-friendly tools available for that.
 
 Gian Luca Brunetti
 
