@@ -1,8 +1,8 @@
 package Sim::OPT::Report;
-# Copyright (C) 2008-2014 by Gian Luca Brunetti and Politecnico di Milano.
+# Copyright (C) 2008-2015 by Gian Luca Brunetti and Politecnico di Milano.
 # This is the module Sim::OPT::Retrieve of Sim::OPT, a program for detailed metadesign managing parametric explorations through the ESP-r building performance simulation platform and performing optimization by block coordinate descent.
 # This is free software.  You can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
-
+ 
 use v5.14;
 # use v5.20;
 use Exporter;
@@ -36,7 +36,7 @@ no warnings;
 
 our @EXPORT = qw( retrieve report get_files );
 
-$VERSION = '0.46.0'; # our $VERSION = '';
+$VERSION = '0.47'; # our $VERSION = '';
 
 
 #########################################################################################
@@ -73,7 +73,7 @@ sub retrieve
 	
 	$tee = new IO::Tee(\*STDOUT, ">>$toshell"); # GLOBAL ZZZ
 	
-	open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
+	#open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
 	open( TOSHELL, ">>$toshell" ) or die "Can't open $toshell: $!"; 
 	say "\nNow in Sim::OPT::Report::retrieve.\n";
 	say $tee "\n#Now in Sim::OPT::Report_retrieve.\n";
@@ -285,7 +285,7 @@ ZZZ
 
 				sub retrieve_stats_results
 				{	
-					my ( $result, $resfile, $retrdata_ref, $reporttitle, $themereport, $counttheme, $countreport, $retfile, $semaphorego, $semaphorestop, $textpattern) = @_;
+					my ( $result, $resfile, $retrdata_ref, $reporttitle, $themereport, $counttheme, $countreport, $retfile, $semaphorego1, $semaphorego2, $semaphorestop1, $semaphorestop2, $textpattern, $afterlines ) = @_;
 					print $tee " IN RETRIEVE STATS: \$sim: $sim, \$resfile: $resfile, \$result: $result, \$reporttitle: $reporttitle, \$themereport: $themereport, \$counttheme: $counttheme, \$countreport, $countreport, \$retfile : $retfile \n";
 					print $tee "\%retrievedata: " . dump( %retrievedata ); 
 					my @retrdata = @$retrdata_ref; say $tee "\@retrdata " . dump(@retrdata);
@@ -367,7 +367,6 @@ TTT
 						{	
 							#say $tee "\$retrievedatum " . dump($retrievedatum);
 							my $reportdata_ref_ref = $reportdata{$counttool}->[$counttheme]; #say $tee "\$reportdata_ref_ref " . dump($reportdata_ref_ref);
-							my $retrievedatarefsdeep = $retrievedata{$counttool}[$counttheme]; #say $tee "\$retrievedatarefsdeep " . dump($retrievedatarefsdeep);
 							my @retrievedatarefs = @{$retrievedatum}; #say $tee "\@retrievedatarefs " . dump(@retrievedatarefs);
 							my $simtitle = $simtitles{$counttool}->[ $counttheme ][0]; #say $tee "\$simtitle " . dump($simtitle);
 							my @sims = @{ $simtitles{$counttool}->[ $counttheme ] }[1..4]; #say $tee "\@sims " . dump(@sims);
@@ -423,9 +422,12 @@ TTT
 										my %datarep = %$item; say $tee "\%datarep" . dump(%datarep); 
 										my $reporttitle = $datarep{reporttitle}; say $tee "\$reporttitle! " . dump($reporttitle); ###DDD HERE
 										my $themereport = $datarep{themereport}; say $tee "\$themereport " . dump($themereport); 
-										my $semaphorego = $datarep{semaphorego}; say $tee "\$semaphorego " . dump($semaphorego); 
-										my $semaphorestop = $datarep{semaphorestop}; say $tee "\$semaphorestop " . dump($semaphorestop); 
+										my $semaphorego1 = $datarep{semaphorego1}; say $tee "\$semaphorego1 " . dump($semaphorego1); 
+										my $semaphorego2 = $datarep{semaphorego2}; say $tee "\$semaphorego2 " . dump($semaphorego2); 
+										my $semaphorestop1 = $datarep{semaphorestop1}; say $tee "\$semaphorestop1 " . dump($semaphorestop1); 
+										my $semaphorestop2 = $datarep{semaphorestop2}; say $tee "\$semaphorestop2 " . dump($semaphorestop2); 
 										my $textpattern = $datarep{textpattern}; say $tee "\$textpattern " . dump($textpattern); 
+										my $afterlines = $datarep{afterlines}; say $tee "\$afterlines " . dump($afterlines); 
 										my $retfile = "$resfile-$reporttitle-$themereport.grt"; #say $tee "\$retfile " . dump($retfile); say TOSHELL "\$retfile " . dump($retfile); say TOSHELL "\$retfile " . dump($retfile);
 										
 										$retstruct[$countcase][$countblock][ $countinstance ][$counttheme][$countreport][$countitem][$counttool] = $retfile;
@@ -445,9 +447,12 @@ TTT
 																					retfile => $retfile,
 																					reporttitle => $reporttitle,
 																					themereport => $themereport,
-																					semaphorego => $semaphorego,
-																					semaphorestop => $semaphorestop,
-																					textpattern => $textpattern
+																					semaphorego1 => $semaphorego1,
+																					semaphorego2 => $semaphorego2,
+																					semaphorestop1 => $semaphorestop1,
+																					semaphorestop2 => $semaphorestop2,
+																					textpattern => $textpattern,
+																					afterlines => $afterlines
 																				} );
 										}
 										
@@ -464,7 +469,7 @@ TTT
 										elsif ( ( ( $themereport eq "loads" ) or ( $themereport eq "tempsstats"  ) ) )
 										{ 
 											retrieve_stats_results( $result, $resfile, \@retrdata, $reporttitle, $themereport, $counttheme, 
-														$countreport, $retfile, $semaphorego, $semaphorestop, $textpattern ); 
+														$countreport, $retfile, $semaphorego1, $semaphorego2, $semaphorestop1, $semaphorestop2, $textpattern, $afterlines ); 
 										}
 										$countitem++;
 									}
@@ -473,12 +478,92 @@ TTT
 							}
 							$counttheme++;
 						}
+						$counttool++;
 					}
-				}
+					elsif ( ( $tooltype eq "generic" ) or ( $tooltype eq "energyplus" ) )
+					{	#use warnings; use strict;
+						my @retfiles = @{ $simstruct[$countcase][$countblock][$countinstance][$counttool] }; say $tee "SAY\@retfiles " . dump(@retfiles); 
+						$retstruct[$countcase][$countblock][$countinstance][$counttool]  = $simstruct[$countcase][$countblock][$countinstance][$counttool] ; 
+						my $counttheme = 0;
+						foreach my $retrievedatum ( @{ $retrievedata{$counttool} } )
+						{
+							my $reportdata_ref_ref = $reportdata{$counttool}->[$counttheme]; #say $tee "\$reportdata_ref_ref " . dump($reportdata_ref_ref);
+							my @retrievedatarefs = @{$retrievedatum}; #say $tee "\@retrievedatarefs " . dump(@retrievedatarefs);
+							my $simtitle = $simtitles{$counttool}->[ $counttheme ][0]; #say $tee "\$simtitle " . dump($simtitle);
+							my @sims = @{ $simtitles{$counttool}->[ $counttheme ] }[1..4]; #say $tee "\@sims " . dump(@sims);
+							
+							my $resfile = $resfiles[ $counttheme ]; #say $tee "\HERE\$resfile " . dump($resfile);
+							
+							if ( -e $resfile ) 
+							{
+								
+
+								my @retrdata = @$retrievedataref; #say $tee "\@retrdata " . dump(@retrdata);
 				
-					
+								open( RETLIST, ">>$retlist"); # or die;
+								open( RETBLOCK, ">>$retblock"); # or die;
+																				
+								my $reportdata_ref = $reportdata_ref_ref->[$countreport]; say $tee "\$reportdata_ref" . dump($reportdata_ref); 
+								my @reportdata = @$reportdata_ref;  say $tee "\@reportdata" . dump(@reportdata); 
+								
+								my $retfile = "$resfile"; #say $tee "\$retfile " . dump($retfile); 
+								if ( not ($retfile ~~ @retcases ) )
+								{
+									push ( @retcases, $retfile );
+									say RETLIST "$retfile";
+								}
+								my @provbag;
+								
+								push ( @provbag, $retfile );
+									
+								my $countitem = 0;
+								foreach my $item ( @reportdata )
+								{
+									my %datarep = %$item; say $tee "\%datarep" . dump(%datarep); 
+									my $reporttitle = $datarep{reporttitle}; say $tee "\$reporttitle! " . dump($reporttitle); ###DDD HERE
+									my $themereport = $datarep{themereport}; say $tee "\$themereport " . dump($themereport); 
+									my $semaphorego1 = $datarep{semaphorego1}; say $tee "\$semaphorego1 " . dump($semaphorego1); 
+									my $semaphorego2 = $datarep{semaphorego2}; say $tee "\$semaphorego2 " . dump($semaphorego2); 
+									my $semaphorestop1 = $datarep{semaphorestop1}; say $tee "\$semaphorestop1 " . dump($semaphorestop1); 
+									my $semaphorestop2 = $datarep{semaphorestop2}; say $tee "\$semaphorestop2 " . dump($semaphorestop2); 
+									my $textpattern = $datarep{textpattern}; say $tee "\$textpattern " . dump($textpattern); 
+									my $afterlines = $datarep{afterlines}; say $tee "\$afterlines " . dump($afterlines);  
+									say TOSHELL "\$retfile " . dump($retfile); say TOSHELL "\$retfile " . dump($retfile);
+									
+									print RETBLOCK "$retfile\n";
+	
+									
+									#if ( not ( $retfile ~~ @provbag ) ) 
+									#{
+										
+										push ( @{ $notecases[ $countcase ][ $countblock ][ $countinstance ][ $counttool ] } , 
+																			{
+																				retfile => $retfile,
+																				reporttitle => $reporttitle,
+																				themereport => $themereport,
+																				semaphorego1 => $semaphorego1,
+																				semaphorego2 => $semaphorego2,
+																				semaphorestop1 => $semaphorestop1,
+																				semaphorestop2 => $semaphorestop2,
+																				textpattern => $textpattern,
+																				afterlines => $afterlines
+																			} );
+									#}
+									$countitem++;
+								}
+
+
+							}
+							else  
+							{ 
+								say "A RESULT FILE NAMED $resfile DOES NOT EXIST. EXITING.";
+							}
+							$counttheme++;
+						}
+						$counttool++;
+					}
+				}	
 			}
-			$counttool++;
 		}
 		$countinstance++;
 	}
@@ -488,6 +573,8 @@ TTT
 	close TOSHELL;
 	close RETLIST;
 	close RETBLOCK;
+	say $tee "INEXIT \@retcases " . dump( @retcases ); 
+	say $tee "\@retstruct " . dump( @retstruct ); 
 	say $tee "\@notecases " . dump( @notecases ); 
 	return ( \@retcases, \@retstruct, \@notecases );
 }	# END SUB RETRIEVE
@@ -522,7 +609,7 @@ sub report # This function retrieves the results of interest from the texts file
 	
 	$tee = new IO::Tee(\*STDOUT, ">>$toshell"); # GLOBAL ZZZ
 	
-	open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
+	#open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
 	open( TOSHELL, ">>$toshell" ) or die "Can't open $toshell: $!";  
 	say "\nNow in Sim::OPT::Report::report.\n";
 	say TOSHELL "\n#Now in Sim::OPT::Report::report.\n";
@@ -552,14 +639,14 @@ sub report # This function retrieves the results of interest from the texts file
 	my @INREPORT = @{ $dirfiles{morphcases} };
 	my @morphstruct = @{ $dirfiles{morphstruct} };
 	my @retcases = @{ $dirfiles{retcases} }; say TOSHELL "dumpINREPORT::report(\@retcases): " . dump(@retcases); 
-	my @retstruct = @{ $dirfiles{retstruct} }; say TOSHELL "dump(\@retstruct): " . dump(@retstruct);
+	my @retstruct = @{ $dirfiles{retstruct} }; say TOSHELL "dumpINREPORT::report(\@retstruct): " . dump(@retstruct);
 	my @repcases = @{ $dirfiles{repcases} };
 	my @repstruct = @{ $dirfiles{repstruct} };
 	my @mergecases = @{ $dirfiles{mergecases} };
 	my @mergestruct = @{ $dirfiles{mergestruct} };
 	my @descendcases = @{ $dirfiles{descendcases} };
 	my @descendstruct = @{ $dirfiles{descendstruct} };
-	my @notecases = @{ $dirfiles{notecases} }; say TOSHELL "dump(\@notecases): " . dump(@notecases);
+	my @notecases = @{ $dirfiles{notecases} }; say TOSHELL "dumpINREPORT::report(\@notecases): " . dump(@notecases);
 	
 	my $morphlist = $dirfiles{morphlist}; #say "dump(\$dat{morphlist}): " . dump($dat{morphlist});
 	my $morphblock = $dirfiles{morphblock};
@@ -604,8 +691,8 @@ sub report # This function retrieves the results of interest from the texts file
 
 		say TOSHELL "#Processing reports for case " . ($countcase + 1) . ", block " . ($countblock + 1);
 		
-		open( REPLIST, ">>$replist" ) or die; say TOSHELL "dump---(\$replist): " . dump($replist) . "\n\n";  
-		open( REPBLOCK, ">>$repblock" ) or die; say TOSHELL "dump---(\$repblock): " . dump($repblock) . "\n\n";
+		open( REPLIST, ">>$replist" ) or die( "$!" ); say TOSHELL "dump---(\$replist): " . dump($replist) . "\n\n";  
+		open( REPBLOCK, ">>$repblock" ) or die( "$!" ); say TOSHELL "dump---(\$repblock): " . dump($repblock) . "\n\n";
 		
 		$repfile = "$mypath/$file-report-$countcase-$countblock.txt";  say TOSHELL "dump---(\$repfile): " . dump($repfile) . "\n\n";
 		open( REPFILE, ">>$repfile") or die "Can't open $repfile $!";
@@ -639,48 +726,81 @@ sub report # This function retrieves the results of interest from the texts file
 					my $retfile = $retitem{retfile}; say TOSHELL "dump---(\$retfile): " . dump($retfile) . "\n\n";  
 					my $reporttitle = $retitem{reporttitle}; say TOSHELL "dump---(\$reporttitle): " . dump($reporttitle) . "\n\n";  
 					my $themereport = $retitem{themereport}; say TOSHELL "dump---(\$themereport): " . dump($themereport) . "\n\n";  
-					my $semaphorego = $retitem{semaphorego}; say TOSHELL "dump---(\$semaphorego): " . dump($semaphorego) . "\n\n";  
-					my $semaphorestop = $retitem{semaphorestop}; say TOSHELL "dump---(\$semaphorestop): " . dump($semaphorestop) . "\n\n";  
+					my $semaphorego1 = $retitem{semaphorego1}; say TOSHELL "dump---(\$semaphorego1): " . dump($semaphorego1) . "\n\n";  
+					my $semaphorego2 = $retitem{semaphorego2}; say TOSHELL "dump---(\$semaphorego2): " . dump($semaphorego2) . "\n\n";  
+					my $semaphorestop1 = $retitem{semaphorestop1}; say TOSHELL "dump---(\$semaphorestop1): " . dump($semaphorestop1) . "\n\n";
+					my $semaphorestop2 = $retitem{semaphorestop2}; say TOSHELL "dump---(\$semaphorestop2): " . dump($semaphorestop2) . "\n\n";  
 					my $textpattern = $retitem{textpattern}; say TOSHELL "dump---(\$textpattern): " . dump($textpattern) . "\n\n";  
-					my $semaphore;
+					my $afterlines = $retitem{afterlines}; say TOSHELL "dump---(\$afterlines): " . dump($afterlines) . "\n\n";  
+					my ( $semaphore1, $semaphore2 );
 					
 					
 					if ( -e $retfile )
 					#if ( not ( eval ( $skipreport ) ) )
 					{
-
-						open( RETFILE, "$retfile" ) or die;
+						open( RETFILE, "$retfile" ) or die( "$!" );
 						my @lines = <RETFILE>;
 						close RETFILE;
-						foreach my $line (@lines)
+						say TOSHELL "RETFILE! " ;
+						my $countline = 0;
+						foreach my $line ( @lines )
 						{
 							$line =~ s/^\s+//;
 							chomp $line;
 							$line = $line . " ";
-							say "\$line: $line";
-							if ( ( not ( $semaphorego ) ) or ( $line =~ m/^$semaphorego/ ) )
+							#say TOSHELL "\$line: $line";
+							
+							if ( ( not defined( $semaphorego1 ) ) or ( $line =~ m/$semaphorego1/ ) )
 							{
-								$semaphore = "on";
+								$semaphore1 = "on";  say TOSHELL "ON1: ";
 							}
-							elsif ( $line =~ m/^$semaphorestop/ )
+							
+							if ( $semaphore1 eq "on" )
 							{
-								$semaphore = "off";
+								if ( ( not defined( $semaphorego2 ) ) or ( $line =~ m/$semaphorego2/ ) )
+								{
+									$semaphore2 = "on"; say TOSHELL "ON2: ";
+								}
 							}
-							if ( ( $line =~ m/^$textpattern/ ) and ( $semaphore eq "on" ) )
+														
+							if ( ( $line =~ m/$semaphorestop1/ ) and ( $semaphorestop1 ne "" ) )
 							{
-								print "HIT! "; say $line;
-								push ( @{ $mergestruct[$countcase][$countblock][ $countinstance ] }, "$retfile $line" );
+								$semaphore1 = "off"; say TOSHELL "OFF1: ";
+							}
+							
+							if ( ( $line =~ m/$semaphorestop2/ ) and ( $semaphorestop2 ne "" ) )
+							{
+								$semaphore2 = "off"; say TOSHELL "OFF2: ";
+							}
+							
+							if ( ( $textpattern ne "" ) and ( $line =~ m/^$textpattern/ ) and ( $semaphore1 eq "on" ) and ( $semaphore2 eq "on" ) )
+							{
+								print TOSHELL "HIT REPORT1! "; say TOSHELL $line;
+								push ( @{ $mergestruct[$countcase][$countblock][ $countinstance ] }, "$retfile $semaphorego1 $semaphorego2 $line " );
+								$semaphore1 = "off";
+								$semaphore2 = "off";
+							}
+							
+							if ( ( $afterlines ne "" ) and ( $semaphore1 eq "on" ) and ( $semaphore2 eq "on" ) and ( $countline == $afterlines ) )
+							{
+								print TOSHELL "HIT REPORT2! "; say TOSHELL $line;
+								push ( @{ $mergestruct[$countcase][$countblock][ $countinstance ] }, "$retfile $semaphorestop1 $semaphorestop2 $line " );
+								$countlines = 0;
+								$semaphore1 = "off";
+								$semaphore2 = "off";
+							}
+							
+							if ( ( $semaphore1 eq "on" ) and ( $semaphore2 eq "on" ) )
+							{
+								$countline++; say TOSHELL "ON3 COUNTLINE: $countline";
 							}
 						}
 					}
 					
 				}
-				#print REPFILE "\n";
-
 			}
 			$counttool++;
 		}
-	
 		say TOSHELL "\@mergestruct: " . dump(@mergestruct); 
 		$countinstance++;
 	}
@@ -689,7 +809,7 @@ sub report # This function retrieves the results of interest from the texts file
 	#{ 
 	#	push ( @mergecases,  @repfilemem );
 	#}
-	say TOSHELL "\@repfilemem ~~ \@mergecases.";
+	#say TOSHELL "\@repfilemem ~~ \@mergecases.";
 	
 	say "#Reporting results for case " . ($countcase + 1) . ", block " . ($countblock + 1) . ", parameter $countvar at iteration $countstep. Instance $countinstance: writing $repfile. ";
 		
@@ -703,7 +823,8 @@ sub report # This function retrieves the results of interest from the texts file
 	}
 	close REPFILE;
 		
-	say TOSHELL "\$repfile: " . dump($repfile);
+	say TOSHELL "HEREIS \@repcases " . dump(@repcases); say TOSHELL "\@repstruct " . dump(@repstruct); say TOSHELL "\@mergestruct " . dump(@mergestruct);
+	say TOSHELL "\@mergecases " . dump(@mergecases); say TOSHELL "\$repfile: " . dump($repfile);
 	close TOSHELL;
 	close OUTFILE;
 	return ( \@repcases, \@repstruct, \@mergestruct, \@mergecases, $repfile ); 
